using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Lexer
{

    public class LexerException : System.Exception
    {
        public LexerException(string msg)
            : base(msg)
        {
        }

    }

    public class Lexer
    {

        protected int position;
        protected char currentCh; //   
        protected int currentCharValue; //     
        protected System.IO.StringReader inputReader;
        protected string inputString;

        public Lexer(string input)
        {
            inputReader = new System.IO.StringReader(input);
            inputString = input;
        }

        public void Error()
        {
            System.Text.StringBuilder o = new System.Text.StringBuilder();
            o.Append(inputString + '\n');
            o.Append(new System.String(' ', position - 1) + "^\n");
            o.AppendFormat("Error in symbol {0}", currentCh);
            throw new LexerException(o.ToString());
        }

        protected void NextCh()
        {
            this.currentCharValue = this.inputReader.Read();
            this.currentCh = (char) currentCharValue;
            this.position += 1;
        }

        public virtual bool Parse()
        {
            return true;
        }
    }

    public class IntLexer : Lexer
    {

        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            int sign = 1;

            if (currentCh == '+' || currentCh == '-')
            {
                if (currentCh == '-')
                    sign = -1;
                NextCh();
            }
        
            if (char.IsDigit(currentCh))
            {
                parseResult = currentCh - '0';
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
                parseResult = parseResult * 10 + (currentCh - '0');
                NextCh();
            }


            if (currentCharValue != -1)
            {
                Error();
            }

            parseResult *= sign;

            return true;

        }
    }
    
    public class IdentLexer : Lexer
    {
        private string parseResult;
        protected StringBuilder builder;
    
        public string ParseResult
        {
            get { return parseResult; }
        }
    
        public IdentLexer(string input) : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            if (char.IsLetter(currentCh))
            {
                builder.Append(currentCh);
                NextCh();
            }
            else
                Error();

            while (currentCharValue > 0)
            {
                if (currentCh == '_' || char.IsLetter(currentCh) || char.IsDigit(currentCh))
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else
                    Error();
            }

            parseResult = builder.ToString();
            return true;

        }

    }

    public class IntNoZeroLexer : IntLexer
    {
        public IntNoZeroLexer(string input)
            : base(input)
        {
        }

        public override bool Parse()
        {
            NextCh();

            if (currentCh == '+' || currentCh == '-')
                NextCh();

            if (!char.IsDigit(currentCh) || currentCh == '0')
                Error();

            while (char.IsDigit(currentCh))
                NextCh();

            return true;
        }
    }

    public class LetterDigitLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            
            NextCh();
            

            if (char.IsLetter(currentCh))
            {
                builder.Append(currentCh);
                char predCh = currentCh;
                NextCh();
                
                while (currentCharValue > 0)
                {
                    if (char.IsLetter(currentCh) && char.IsDigit(predCh) || char.IsLetter(predCh) && char.IsDigit(currentCh))
                    {
                        predCh = currentCh;
                        builder.Append(currentCh);
                        NextCh();
                    }
                    else
                        Error();
                }
            }
            else
            {
                Error();
            }

            parseResult = builder.ToString();
            return true;
        }

    }

    public class LetterListLexer : Lexer
    {
        protected List<char> parseResult;

        public List<char> ParseResult
        {
            get { return parseResult; }
        }

        public LetterListLexer(string input)
            : base(input)
        {
            parseResult = new List<char>();
        }

        public override bool Parse()
        {
            NextCh();

            if (char.IsLetter(currentCh))
            {
                parseResult.Add(currentCh);
                NextCh();
            } 
            else
                Error();

            while (currentCh == ',' || currentCh == ';')
            {
                NextCh();
                if (char.IsLetter(currentCh))
                    parseResult.Add(currentCh);
                else
                    Error();
                NextCh();
            }

            return true;

        }
    }

    public class DigitListLexer : Lexer
    {
        protected List<int> parseResult;

        public List<int> ParseResult
        {
            get { return parseResult; }
        }

        public DigitListLexer(string input)
            : base(input)
        {
            parseResult = new List<int>();
        }

        public override bool Parse()
        {
            NextCh();

            if (char.IsDigit(currentCh))
            {
                parseResult.Add(currentCh - '0');
                NextCh();
            }
            else
                Error();

            if (currentCharValue > 0 && currentCh != ' ')
                Error();

            while (currentCh == ' ')
            {
                NextCh();
                if (char.IsDigit(currentCh))
                    parseResult.Add(currentCh - '0');
                else if (currentCh == ' ')
                    continue;
                else
                    Error();
                NextCh();
            }

            return true;
        }
    }

    public class LetterDigitGroupLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }
        
        public LetterDigitGroupLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if (currentCharValue == -1)
            {
                Error();
            }

            while (currentCharValue > 0)
            {
                if (char.IsLetter(currentCh))
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else
                    Error();

                if (char.IsLetter(currentCh))
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else if (currentCharValue <= 0)
                    break;
                if (char.IsDigit(currentCh))
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else if (currentCharValue <= 0)
                    break;
                else
                    Error();
                if (char.IsDigit(currentCh))
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else if (currentCharValue <= 0)
                    break;
            }

            parseResult = builder.ToString();

            return true;
        }
       
    }

    public class DoubleLexer : Lexer
    {
        private StringBuilder builder;
        private double parseResult;

        public double ParseResult
        {
            get { return parseResult; }

        }

        public DoubleLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            if (currentCharValue < 0 || currentCh == '.')
                Error();

            parseResult = 0;
            double doubleCoef = 0.1;

            for (int i = 0; i < 2; i++)
            {
                while (char.IsDigit(currentCh))
                {
                    if (i == 0)
                        parseResult = parseResult * 10 + (currentCh - '0');
                    else 
                    {
                        parseResult += (currentCh - '0') * doubleCoef;
                        doubleCoef /= 10;
                    }
                    NextCh();

                }
                if (currentCh == '.')
                {
                    NextCh();
                    if (currentCharValue < 0)
                        Error();
                }
                
            }


            return true;

        }

    }

    public class StringLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public StringLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if (currentCh == '\'')
            {
                builder.Append(currentCh);
                NextCh();
            }
            else
                Error();

            while (currentCh != '\'' && currentCharValue > 0)
            {
                builder.Append(currentCh);
                NextCh();
            }

            if (currentCh == '\'')
            {
                builder.Append(currentCh);
                NextCh();
            }
            else
                Error();

            if (currentCharValue > 0)
                Error();

            parseResult = builder.ToString();

            return true;
        }
    }

    public class CommentLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public CommentLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            bool end = false;
            NextCh();

            if (currentCh == '/')
            {
                builder.Append(currentCh);
                NextCh();
                if(currentCh == '*')
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else
                    Error();
            }
            else
                Error();

            if (currentCharValue <= 0)
                Error();

            while (currentCharValue > 0 && !end)
            {
                if (currentCh == '*')
                {
                    builder.Append(currentCh);
                    NextCh();
                    if (currentCh == '/')
                    {
                        builder.Append(currentCh);
                        NextCh();
                        end = true;
                    }
                }
                else
                {
                    builder.Append(currentCh);
                    NextCh();
                }
            }

            if(currentCharValue > 0 || !end)
                Error();

            parseResult = builder.ToString();

            return true;

        }
    }

    public class IdentChainLexer : Lexer
    {
        private StringBuilder builder;
        private List<string> parseResult;

        public List<string> ParseResult
        {
            get { return parseResult; }

        }

        public IdentChainLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
            parseResult = new List<string>();
        }

        public override bool Parse()
        {
            //parse ident
            NextCh();

            if (currentCharValue <= 0)
                Error();

            while (currentCharValue > 0)
            {
                if (char.IsLetter(currentCh))
                {
                    NextCh();
                }
                else
                    Error();

                while (currentCh == '_' || char.IsLetter(currentCh) || char.IsDigit(currentCh))
                {
                    NextCh();
                }
                if (currentCh == '.')
                {
                    NextCh();
                    if (currentCharValue <= 0)
                        Error();
                }
                else if (currentCharValue <= 0)
                    return true;
                else
                    Error();
            }
            
            return true;
            //
        }
    }

    public class Program
    {
        public static void Main()
        {
            string input = "154216";
            Lexer L = new IntLexer(input);
            try
            {
                L.Parse();
            }
            catch (LexerException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}